package com.qmth.themis.business.listener;

import com.alibaba.fastjson.JSONObject;
import com.qmth.themis.business.constant.SystemConstant;
import com.qmth.themis.business.entity.TBSession;
import com.qmth.themis.business.service.TBSessionService;
import com.qmth.themis.business.util.RedisUtil;
import com.qmth.themis.common.contanst.Constants;
import org.apache.rocketmq.client.consumer.DefaultMQPushConsumer;
import org.apache.rocketmq.client.consumer.listener.ConsumeConcurrentlyContext;
import org.apache.rocketmq.client.consumer.listener.ConsumeConcurrentlyStatus;
import org.apache.rocketmq.client.consumer.listener.MessageListenerConcurrently;
import org.apache.rocketmq.common.message.Message;
import org.apache.rocketmq.common.message.MessageExt;
import org.apache.rocketmq.spring.annotation.RocketMQMessageListener;
import org.apache.rocketmq.spring.annotation.SelectorType;
import org.apache.rocketmq.spring.core.RocketMQListener;
import org.apache.rocketmq.spring.core.RocketMQPushConsumerLifecycleListener;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.List;
import java.util.Map;
import java.util.Objects;

import static com.alibaba.fastjson.JSONObject.parseObject;

/**
 * @Description: 普通消息监听
 * @Param:
 * @return:
 * @Author: wangliang
 * @Date: 2020/6/28
 */
@Service
public class RocketConsumer implements MessageListenerConcurrently {

    private final static Logger log = LoggerFactory.getLogger(RocketConsumer.class);

    @Resource
    TBSessionService tbSessionService;

    @Resource
    RedisUtil redisUtil;

    @Override
    public ConsumeConcurrentlyStatus consumeMessage(List<MessageExt> msgs, ConsumeConcurrentlyContext consumeConcurrentlyContext) {
        try {
            for (MessageExt messageExt : msgs) {
                log.info("sessionConsumer重试次数:{}", messageExt.getReconsumeTimes());
                JSONObject jsonObject = parseObject(new String(messageExt.getBody(), Constants.CHARSET));
                log.info("sessionConsumer接受到的消息:{}", jsonObject.toJSONString());
                boolean waitStoreMsgOK = jsonObject.getBoolean("waitStoreMsgOK");
                if (waitStoreMsgOK) {
                    Map map = (Map) jsonObject.get(SystemConstant.PROPERTIES);
                    String sessionId = (String.valueOf(map.get(String.valueOf(map.get("TAGS")))));
                    if (Objects.nonNull(sessionId)) {
                        TBSession tbSession = (TBSession) redisUtil.getSessionTopicList(sessionId);
                        if (Objects.nonNull(tbSession.getAck()) && tbSession.getAck().intValue() != SystemConstant.STANDARD_ACK_TYPE) {
                            tbSessionService.saveSessionInfo(tbSession);
                            tbSession.setAck(SystemConstant.STANDARD_ACK_TYPE);
                            redisUtil.setUserSession(tbSession.getId(), tbSession);
                            redisUtil.deleteSessionTopicList(tbSession.getId());
                        }
                    }
                    return ConsumeConcurrentlyStatus.CONSUME_SUCCESS;//成功
                } else {
                    return ConsumeConcurrentlyStatus.RECONSUME_LATER;//重试
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
            return ConsumeConcurrentlyStatus.RECONSUME_LATER;//重试
        }
        return ConsumeConcurrentlyStatus.CONSUME_SUCCESS;//成功
    }

    @Service
    @RocketMQMessageListener(consumerGroup = "${mq.config.sessionConsumerGroup}", topic = "${mq.config.sessionTopic}", selectorType = SelectorType.TAG, selectorExpression = "${mq.config.sessionTopicTag}")
    public class sessionConsumer implements RocketMQListener<Message>, RocketMQPushConsumerLifecycleListener {

        @Override
        public void onMessage(Message message) {
            //实现RocketMQPushConsumerLifecycleListener监听器之后，此方法不调用
        }

        @Override
        public void prepareStart(DefaultMQPushConsumer defaultMQPushConsumer) {
            defaultMQPushConsumer.setConsumeMessageBatchMaxSize(SystemConstant.CONSUME_MESSAGE_BATCH_MAX_SIZE);//每次拉取10条
            defaultMQPushConsumer.setMaxReconsumeTimes(SystemConstant.MAXRECONSUMETIMES);//最大重试次数
            defaultMQPushConsumer.registerMessageListener(RocketConsumer.this::consumeMessage);
        }
    }

    @Service
    @RocketMQMessageListener(consumerGroup = "${mq.config.sessionConsumerGroupDlq}", topic = "${mq.config.sessionTopicDlq}", selectorType = SelectorType.TAG, selectorExpression = "${mq.config.sessionTopicTag}")
    public class dlqSessionConsumer implements RocketMQListener<Message>, RocketMQPushConsumerLifecycleListener {

        @Override
        public void onMessage(Message message) {
            //实现RocketMQPushConsumerLifecycleListener监听器之后，此方法不调用
        }

        @Override
        public void prepareStart(DefaultMQPushConsumer defaultMQPushConsumer) {
            log.info("dlqSessionConsumer死信队列进来了");
            defaultMQPushConsumer.setConsumeMessageBatchMaxSize(SystemConstant.CONSUME_MESSAGE_BATCH_MAX_SIZE);//每次拉取10条
            defaultMQPushConsumer.setMaxReconsumeTimes(SystemConstant.MAXRECONSUMETIMES);//最大重试次数
            defaultMQPushConsumer.registerMessageListener(RocketConsumer.this::consumeMessage);
        }
    }
}
